# Copyright (C) 2003, 2004
# Emmanuel Saracco <esaracco@noos.fr>
# Easter-eggs <http://www.easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307, USA.

do '../web-lib.pl';
&init_config();

use POSIX;
use File::Basename;
use Date::Manip;

# min stable clamav version supported by this module
my $supported_versions = '0.70';

# min devel (CVS) clamav version supported by this module
my $min_devel_date = '20040417';

# max items to display/page for quarantine search
# result
my $PAGE_ITEMS = 50;

# freshclam configuration
my %freshclam_config = ();

# freshclam predefined variables
# name => [1 = need value | 0 = alone]
my %freshclam_predefined = (
  'DatabaseOwner' => 1,
  'Checks' => 1,
  'UpdateLogFile' => 1,
  'DatabaseMirror' => 1,
  'MaxAttempts' => 1,
  'HTTPProxyServer' => 1,
  'HTTPProxyPort' => 1,
  'HTTPProxyUsername' => 1,
  'HTTPProxyPassword' => 1,
  'NotifyClamd' => 0,
  'OnUpdateExecute' => 1,
  'OnErrorExecute' => 1
);

# clamav configuration
my %clamav_config = ();

# clamav predefined variables
# name => [1 = need value | 0 = alone]
my %clamav_predefined = (
  'LogFile' => 1,
  'LogFileUnlock' => 0,
  'LogFileMaxSize' => 1,
  'LogTime' => 0,
  'LogSyslog' => 0,
  'PidFile' => 1,
  'DataDirectory' => 1,
  'LocalSocket' => 1,
  'FixStaleSocket' => 0,
  'TCPSocket' => 1,
  'TCPAddr' => 1,
  'MaxConnectionQueueLength' => 1,
  'MaxThreads' => 1,
  'ReadTimeout' => 1,
  'MaxDirectoryRecursion' => 1,
  'FollowDirectorySymlinks' => 0,
  'FollowFileSymlinks' => 0,
  'SelfCheck' => 1,
  'VirusEvent' => 1,
  'User' => 1,
  'AllowSupplementaryGroups' => 0,
  'Foreground' => 0,
  'Debug' => 0,
  'StreamSaveToDisk' => 0,
  'StreamMaxLength' => 1,
  'ScanMail' => 0,
  'ScanArchive' => 0,
  'ScanRAR' => 0,
  'ArchiveDetectEncrypted' => 0,
  'ArchiveBlockEncrypted' => 0,
  'ArchiveMaxFileSize' => 1,
  'ArchiveMaxRecursion' => 1,
  'ArchiveMaxFiles' => 1,
  'ArchiveLimitMemoryUsage' => 0,
  'ClamukoScanOnLine' => 0,
  'ClamukoScanOnOpen' => 0,
  'ClamukoScanOnClose' => 0,
  'ClamukoScanOnExec' => 0,
  'ClamukoIncludePath' => 1,
  'ClamukoExcludePath' => 0,
  'ClamukoMaxFileSize' => 1,
  'ClamukoScanArchive' => 0
);

# week days
my @days = (
  $text{'SUNDAY'},
  $text{'MONDAY'},
  $text{'TUESDAY'},
  $text{'WEDNESDAY'},
  $text{'THURSDAY'},
  $text{'FRIDAY'},
  $text{'SATURDAY'}
);

# clamav_is_amavis_ng ()
# IN: -
# OUT: TRUE if amavis-ng is installed
#
# Check if amavis-ng is installed
# 
sub clamav_is_amavis_ng ()
{
  return ($all_path{'amavis-ng'} ne '');
}

# clamav_is_amavisd_new ()
# IN: -
# OUT: TRUE if amavisd-new is installed
#
# Check if amavisd-new is installed
#
sub clamav_is_amavisd_new ()
{
  return ($all_path{'amavisd-new'} ne '' or $all_path{'amavisd'} ne '');
}

# all binaries pathes are here
my $all_path = ();
&clamav_init_all_path ();

# clamav_check_config ()
# IN: -
# OUT: -
#
# check if the module config is ok. if not, display a warning
# and exit
# 
sub clamav_main_check_config
{
  my $ok = 0;
  my $msg = '';
    
  # clean config inputs
  &clamav_trim_config ();
  
  # touch log files
  if ($config{'clamav_clamav_log'} ne '')
  {
    system ("$all_path{'touch'}touch $config{'clamav_clamav_log'}") ;
    system ("$all_path{'chmod'}chmod clamav.clamav $config{'clamav_clamav_log'}");
  }
  if ($config{'clamav_freshclam_log'} ne '')
  {
    system ("$all_path{'touch'}touch $config{'clamav_freshclam_log'}");
    system ("$all_path{'chmod'}chmod clamav.clamav $config{'clamav_freshclam_log'}");
  }
  
  # do tests
  if (not &clamav_check_version ())
  {
     $msg = $text{'MSG_BAD_CLAMAV_VERSION'};
  }
  elsif ($config{'clamav_rcd_path'} eq '' or
    ! -d $config{'clamav_rcd_path'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_RCD_PATH'};
  }
  elsif ($config{'clamav_init_script'} eq '' or
    ! -x $config{'clamav_init_script'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_DAEMON'};
  }
  elsif ($config{'clamav_daemon_name'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_DAEMON_NAME'};
  }
  elsif ($all_path{'clamscan'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMSCAN'};
  }
  elsif ($config{'clamav_clamav_log'} eq '' or
   ! -f $config{'clamav_clamav_log'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_LOG'};
  }
  elsif ($config{'clamav_clamav_conf'} eq '' or
    ! -f $config{'clamav_clamav_conf'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_CONF'};
  }
  elsif ($config{'clamav_db1'} eq '' or
    ! -f $config{'clamav_db1'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_DB1'};
  }
  elsif ($config{'clamav_db2'} eq '' or
    ! -f $config{'clamav_db2'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_DB2'};
  }
  elsif ($all_path{'freshclam'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_REFRESH'};
  }
  elsif ($config{'clamav_freshclam_log'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_LOG'};
  }
  elsif ($config{'clamav_refresh_use_cron'} == 0)
  {
    if ($config{'clamav_freshclam_conf'} eq '' or
      ! -f $config{'clamav_freshclam_conf'})
    {
      $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_CONF'};
    }
    elsif ($config{'clamav_freshclam_init_script'} eq '' or
      ! -x $config{'clamav_freshclam_init_script'})
    {
      $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_INIT'};
    }
    else
    {
      $ok = 1;
    }
  }
  elsif ($config{'clamav_refresh_use_cron'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_USE_CRON'};
  }
  else
  {
    $ok = 1;
  }

  # if there was a problem, exit displaying
  # a message
  &clamav_check_config_exit ($msg) if (not $ok);

  # create module's temporary directory
  system ("$all_path{'mkdir'}mkdir -p /tmp/.clamav/");
}

# clamav_init_all_path ()
# IN: -
# OUT: -
#
# Initialize binaries pathes
# 
sub clamav_init_all_path
{
  my @exe = (
    'amavis-ng',
    'amavisd-new',
    'amavisd',
    'amavis-inject',
    'chmod',
    'clamscan',
    'freshclam',
    'grep',
    'hexdump',
    'ln',
    'mkdir',
    'mv',
    'cp',
    'runlevel',
    'rm',
    'rsmtp',
    'sendmail',
    'sigtool',
    'strings',
    'touch'
  );

  foreach my $item (@exe)
  {
    $all_path{$item} = &clamav_has_command ($item);
  }
}

# clamav_get_proxy_settings ()
# IN: -
# OUT: A array with 0="proxy server" and 1="proxy port"
#
# Load the proxy settings and return them
# 
sub clamav_get_proxy_settings
{
  &clamav_load_freshclam_config ();

  return (
    "$freshclam_config{'HTTPProxyServer'}",
    "$freshclam_config{'HTTPProxyPort'}"
  )
    if (
      $freshclam_config{'HTTPProxyServer'} and 
      $freshclam_config{'HTTPProxyPort'}
    );

  return ();
}

# clamav_display_combo_freshclam_predefined ( $ )
# IN: 1 if combo must not display values that already exist
#     in the config file
# OUT: 1 if combo box has been displayed (if there is result)
#
# Display a combo box with freshclam predifined variables
# 
sub clamav_display_combo_freshclam_predefined ( $ )
{
  my $uniq = shift;
  my $buf = '';
  my $ok = 0;
  
  $buf = qq(<select name="nsfreshclam_add_key">\n);
  foreach $key (sort keys %freshclam_predefined)
  {
    if (not defined $freshclam_config{$key})
    {
      $ok = 1;
      $buf .= qq(<option value="$key">$key</option>\n)
    }
  }
  $buf .= qq(</select>\n);

  # only print if there is result
  print $buf if ($ok == 1);

  return $ok;
}

# clamav_display_combo_clamav_predefined ( $ )
# IN: 1 if combo must not display values that already exist
#     in the config file
# OUT: 1 if combo box has been displayed (if there is result)
#
# Display a combo box with clamav predifined variables
#
sub clamav_display_combo_clamav_predefined ($)
{
  my $uniq = shift;
  my $buf = '';
  my $ok = 0;
  
  $buf = qq(<select name="nsclamav_add_key">\n);
  foreach $key (sort keys %clamav_predefined)
  {
    if (not defined $clamav_config{$key})
    {
      $ok = 1;
      $buf .= qq(<option value="$key">$key</option>\n)
    }
  }
  $buf .= qq(</select>\n);

  # only print if there is result
  print $buf if ($ok == 1);

  return $ok;
}

# clamav_quarantine_resend_check_config ()
# IN: -
# OUT: -
#
# check if the config is ok for resending quarantined emails. 
# if not, display a warning and exit
# 
sub clamav_quarantine_resend_check_config
{
  my $ok = 0;
  my $msg = '';

  &clamav_trim_config ();
  
  if (($all_path{'rsmtp'} eq '') and ($all_path{'sendmail'} eq ''))
  {
    $msg = $text{'MSG_CONFIG_ALERT_MTA'};
  }
  else
  {
    $ok = 1;
  }

  &clamav_check_config_exit ($msg) if (not $ok);
}

# clamav_has_command ( $ )
# IN: binary to test
# OUT: system path for this binary
#
# Search and return path for a given binary file
# -> clamav_init_all_path () need to be call before
#    this function
# 
sub clamav_has_command
{
  my $exe = shift;
  my @path = (
    '/bin/',
    '/sbin/',
    '/usr/bin/',
    '/usr/sbin/',
    '/usr/local/bin/',
    '/usr/local/sbin/',
    '/opt/bin/',
    '/opt/sbin/',
    '/opt/local/bin/',
    '/opt/local/sbin'
  );

  foreach my $item (@path)
  {
    return $item if (-e $item . $exe);  
  }
  
  return '';
}

# clamav_quarantine_main_check_config ()
# IN: -
# OUT: -
#
# check if the config is ok for managing quarantined emails. 
# if not, display a warning and exit
#
sub clamav_quarantine_main_check_config
{
  my $ok = 0;

  &clamav_trim_config ();
  
  if (not &clamav_is_amavis_ng () and not &clamav_is_amavisd_new ())
  {
    print qq(<p>$text{'MSG_CONFIG_ALERT_AMAVIS'}</p>);
  }
  elsif ($config{'clamav_quarantine'} eq '' or
    ! -d $config{'clamav_quarantine'})
  {
    print qq(<p>$text{'MSG_CONFIG_ALERT_QUARANTINE'}</p>);
  }
  else
  {
    $ok = 1;
  }

  &clamav_check_config_exit ($msg) if (not $ok);
}

# clamav_signatures_check_config ()
# IN: -
# OUT: -
#
# check if the server config is ok for the signatures creation section. 
# if not, display a warning and exit
#
sub clamav_signatures_check_config
{
  &clamav_trim_config ();

  &clamav_check_config_exit ($text{'MSG_ERROR_SIGNATURES_CONFIG'})
    if (
      ($all_path{'strings'} eq '') or 
      ($all_path{'hexdump'} eq '')
    );
}

# clamav_check_config_exit ( $ )
# IN: Message to display
# OUT: -
#
# print a message and exit
# 
sub clamav_check_config_exit ( $ )
{
  my $msg = shift;

  print qq(<p>$msg</p>) if ($msg);
  print qq(<hr>);
  &footer("/", $text{'index'});

  exit (1);
}

# clamav_trim_config ()
# IN: -
# OUT: -
#
# remove trailing and pending spaces from every module's
# config variable
# 
sub clamav_trim_config
{
  foreach $key (keys %config)
  {
    $config{$key} =~ s/^ +//g;
    $config{$key} =~ s/ +$//g;
  }
}

# clamav_html_cut ( $ $ )
# IN: the string to cut, the max length before cutting
# OUT: the cutted string with HTML <br> tags
#
# cut a string with HTML <br> tags
# 
sub clamav_html_cut ( $ $ )
{
  my ($string, $len) = @_;
  my $i = 0;
  my $new = '';

  for ($i = 0; $i < length ($string); $i++)
  {
    $new .= '<br>' if ($i > 0 and not ($i % $len));
    $new .= substr ($string, $i, 1);
  }

  return $new;
}

# clamav_check_signature ( $ )
# IN: new virus signature
# OUT: 1 => the signature begun with 00
#      2 => the signature is not between 40-200 characters
#      3 => signature is ok
#
# check the ClamAV compatibility for a given virus signature
#      
sub clamav_check_signature ( $ )
{
  my $signature = shift;
  my $ret = 3;
  

  if ($signature =~ /^00/)
  {
    $ret = 1;
  }
  elsif (length ($signature) < 40 or length ($signature) > 200)
  {
    $ret = 2;
  }

  return $ret;
}

# clamav_check_version ()
# IN: -
# OUT: boolean value
#
# return true if the installed version of clamav
# is supported by this module. if this is a "stable" release, we look
# at its version number, otherwise if it is a CVS release we look at its
# date
# 
sub clamav_check_version
{
  my $date = '';
  my $out = '';
  my $version = 0;
  my $ret = 0;

  $out = `$all_path{'clamscan'}clamscan --version 2>&1`;
  #$out = "clamscan / ClamAV version devel-20040410";

  if ($out =~ /devel/)
  {
    $date = (split (/-/, $out))[1];
    $ret = (&Date_Cmp ($min_devel_date, $date) <= 0);
  }
  else
  {
    (split (' ', $out))[4] =~ /^(\d+\.\d+).*$/;
    $version = $1;
    $ret = ($version >= $supported_versions);
  }

  return $ret;
}

# clamav_get_runlevel ()
# IN: -
# OUT: the current runlevel
#
# return the current runlevel of the server system
# 
sub clamav_get_runlevel
{
  my $runlevel = '';

  $runlevel = `$all_path{'runlevel'}runlevel`;
  $runlevel =~ s/N //;
  $runlevel =~ s/(^\s+|\s+$)//g;

  $runlevel = int ($runlevel);
  $runlevel = '' if ($runlevel == 0);

  return $runlevel;
}

# clamav_get_freshclam_daemon_settings ()
# IN: -
# OUT: check period
#
# Return the check period specified in the configuration file of
# freshclam daemon
#
sub clamav_get_freshclam_daemon_settings
{
  my $conf = '';
  my $ret = '';
  
  &clamav_load_freshclam_config ();
  
  $ret = $freshclam_config{'Checks'};
  $ret = 1 if $ret eq '';

  return $ret;
}

# clamav_vdb_get_pure_name ( $ )
# IN: ClamAV registered virus name
# OUT: Simple virus name
#
# Try to guess the generic name of the given virus
# 
sub clamav_vdb_get_pure_name
{
  my $name = shift;
  my ($b, $m, $e);

  if ($name =~ /^(.*)[\.\-](.*)[\.\-](.*)$/)
  {
    ($b, $m, $e) = ($1, $2, $3);
    
      return ($m !~ /[a-z,A-Z]/ or length ($m) == 1) ?
        &clamav_vdb_get_pure_name ("$b") : $m;
  }

  if ($name =~ /^(.*)[\.\-](.*)$/)
  {
    ($b, $e) = ($1, $2);

    if ($e !~ /[a-z,A-Z]/) {return $b;} elsif ($e) {return $e;}
  }

  if ($name =~ /^(.*)[\.\-].*$/)
  {
    return $1;
  }

  return $name;
}

# clamav_vdb_search ( $ $ )
# IN: virus name, TRUE if we must do a strict search
# OUT: -
#
# Do a search in ClamAV database and display the result
# 
sub clamav_vdb_search ( $ $ )
{
  my ($virus, $strict, $case, $sortr) = @_;
  my $string = '';
  my $escaped = '';
  my $first = 1;

  exit (1) if ($virus =~ /;/);

  # case sensitive search?
  $case = ($case) ? ' ' : ' -i ';

  # sort results
  $sortr = ($sortr) ? ' sort |' : '';
  
  # strict check (exact match)?
  $string = ($virus ne '') ? 
    (($strict) ? 
      "$all_path{'sigtool'}sigtool --list-sigs | $all_path{'grep'}grep $case \"^$virus\$\" | $sortr" :
      "$all_path{'sigtool'}sigtool --list-sigs | $all_path{'grep'}grep $case $virus | $sortr") :
    "$all_path{'sigtool'}sigtool --list-sigs | $sortr";

  # display results
  open (H, $string);
  while (<H>)
  {
    chomp ();
    $escaped = &urlize (&clamav_vdb_get_pure_name ($_));

    if ($first)
    {
      $first = 0;
      
      print qq(
        <table border=1>
        <tr $tb><th>$text{'NAME'}</th>
        <th>Symantec</th>
        <th>McAfee</th>
        <th>Trend Micro</th>
        </tr>
      );
    }

    $symantec_url = 
      "http://search.symantec.com/custom/us/query.html?" .
      "lk=1&rf=0&qt=$escaped&qp=" . &urlize ("url:/var/log/");
    $mcafee_url = 
      "http://us.mcafee.com/virusInfo/default.asp?" .
      "id=alphar&SearchType=2&searchString=$escaped";
    $trend_url = 
      "http://www.trendmicro.com/vinfo/virusencyclo/default2.asp?" .
      "m=q&virus=$escaped&alt=$escaped";
      
    print qq(
      <tr><td $cb>$_</td>
      <td $cb><a href="$symantec_url">$text{'MORE'}</a></td>
      <td $cb><a href="$mcafee_url">$text{'MORE'}</a></td>
      <td $cb><a href="$trend_url">$text{'MORE'}</a></td>
      </tr>
    );
  }
  close (H);
  
  if (not $first)
  {
    print qq(</table>);
  }
  else
  {
    printf "<p>$text{'NO_RESULT'}</p>", $virus;
  }
}

# clamav_get_db_viruses_count ()
# IN: -
# OUT: Number of viruses in ClamAV database
#
# Count the number of viruses in ClamAV databases and return
# it
# 
sub clamav_get_db_viruses_count
{
  my $count = `$all_path{'sigtool'}sigtool --list-sigs | wc -l`;
  
  $count =~ s/ //g;
  
  return $count;
}

# clamav_get_freshclam_daemon_state ()
# IN: -
# OUT: true if the freshclam daemon is deactivated
#
# Return the actual state of the freshclam daemon service
# 
sub clamav_get_freshclam_daemon_state
{
  my $runlevel = '';
  my $freshclam = '';
  my $freshclambase = '';
  my $link = '';
  my $bsd_clam_state = 'NO';

  $runlevel = &clamav_get_runlevel ();
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
  $bsd_clam_state = &clamav_bsd_get_state ('freshclam')
    if ($gconfig{'os_type'} =~ /bsd/);

  # debian
  if ($gconfig{'os_type'} =~ /debian/)
  {
    return (! -e "/etc/network/if-up.d/$freshclambase"); 
  }
  # bsd
  elsif ($gconfig{'os_type'} =~ /bsd/)
  {
    return ($bsd_clam_state eq 'NO');
  }
  # others
  else
  {
    $link = < $config{'clamav_rcd_path'}/rc$runlevel.d/S*$freshclambase >;
    $link = < $config{'clamav_rcd_path'}/rc.local/S*$freshclambase >
      if ($link eq '');
	
    return ($link eq '');
  }

  return 0;
}

# clamav_bsd_get_state ( $ )
# IN: daemon name (clamav or freshclam)
# OUT: system setting for the daemon (NO or YES)
#
# Return the current system setting for a given daemon for BSD systems
# 
sub clamav_bsd_get_state ( $ )
{
  my $daemon = shift;
  my $ret = 'NO';

  open (H,"/etc/rc.conf");
  while (my $line = <H>)
  {
    if ($line =~ /clamav_${daemon}_enable/)
    {
      chop ($line);
      $ret = (split (/=/, $line))[1];
      $ret =~ s/"//g;
    }
  }
  close (H);

  return $ret;
}


# clamav_verif_refresh_method_ok ()
# IN: -
# OUT: 1 => config say to use a daemon but no daemon exist on the system
#      2 => config say to use a daemon, but a cron exist on the system
#      3 => config say tu use cron, but a daemon exist on the system
#      4 => config and system are ok
#
# Check if the refresh method choose by user in module config is
# ok according to the system
# 
sub clamav_verif_refresh_method_ok
{
  my $runlevel = '';
  my $freshclam = '';
  my $freshclambase = '';
  my $link = '';
  my $use_cron = 0;
  my $bsd_clam_state = 'NO';
  
  $runlevel = &clamav_get_runlevel ();
  $use_cron = $config{'clamav_refresh_use_cron'};
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
  $bsd_clam_state = &clamav_bsd_get_state ('freshclam')
      if ($gconfig{'os_type'} =~ /bsd/);
   
  # debian | bsd
  if ($gconfig{'os_type'} =~ /debian/ or
    $gconfig{'os_type'} =~ /bsd/)
  {
    return 1
      if (($use_cron == 0) and 
        (! -e "$freshclam"));
  }
  # others
  else
  {
    $link = < $config{'clamav_rcd_path'}/rc$runlevel.d/{S,K}*$freshclambase >;
    $link = < $config{'clamav_rcd_path'}/rc.local/{S,K}*$freshclambase >
      if ($link eq '');

   return 1
    if (($use_cron == 0) and ($link eq ''));
  }

  return 2
    if (($use_cron == 0) and (&clamav_get_cron_settings ('update')));

  # debian
  if ($gconfig{'os_type'} =~ /debian/)
  {
    return 3
      if (($use_cron == 1) and (-e < /etc/network/if-up.d/$freshclambase >));
  }
  # bsd
  elsif ($gconfig{'os_type'} =~ /bsd/)
  {
    return 3
      if (($use_cron == 1) and $bsd_clam_state eq 'YES');
  }
  else
  {
    $link = < $config{'clamav_rcd_path'}/rc$runlevel.d/S*$freshclambase >;
    $link = < $config{'clamav_rcd_path'}/rc.local/S*$freshclambase >
      if ($link eq '');

    return 3
      if (($use_cron == 1) and ($link ne ''));
  }

  return 4;
}

# clamav_scandir ( $ $ )
# IN: directory to scan, flag to indicate to use or not recursivity
# OUT: -
#
# Check a given directory and display freshclam output on the current web
# page
# 
sub clamav_scandir ( $ $ )
{
  my ($dir, $recursive) = @_;
 
  $recursive = ($recursive) ?
    ' -r ' : '';
      
  print qq(<div><pre style="background: #cccccc">);
  open (H, "$all_path{'clamscan'}clamscan $recursive $dir 2>&1 |");
  while (<H>)
  {
    s/^*\n$/<br>/g;
    print;
  }
  close (H);
  print qq(</pre>);
}

# clamav_get_last_db_update ()
# IN: -
# OUT: a array with main and daily db update date
#
# Check the signatures databases dates and return them in a
# array
# 
sub clamav_get_last_db_update
{
  my $main = '';
  my $daily = '';
         
  $main = &make_date ((stat($config{'clamav_db1'}))[9])
      if (-e $config{'clamav_db1'});

  $daily = &make_date ((stat($config{'clamav_db2'}))[9])
      if (-e $config{'clamav_db2'});

  return ($main, $daily);
}

# clamav_set_db_no_autoupdate ()
# IN: -
# OUT: -
#
# if user choose to not automatically update db, we stop clamd and
# deactivate cron and services
# 
sub clamav_set_db_no_autoupdate
{
  my $runlevel = '';
  my $filename = '';
  my $freshclam = '';
  my $freshclambase = '';
  my $type_other = 0;
  my $order = 0;

  $runlevel = &clamav_get_runlevel ();
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
  system ("$freshclam stop");

  # debian
  if ($gconfig{'os_type'} =~ /debian/)
  {
    system ("$all_path{'rm'}rm -f /etc/network/if-up.d/$freshclambase");
    system ("$all_path{'rm'}rm -f /etc/network/if-down.d/$freshclambase");
  }
  # bsd
  elsif ($gconfig{'os_type'} =~ /bsd/)
  {
    system ("$config{'clamav_freshclam_init_script'} stop");
    &clamav_bsd_update_state ('freshclam', 'NO');
  }
  # others
  else
  {
    $filename = 
      < $config{'clamav_rcd_path'}/rc$runlevel.d/{K,S}*$freshclambase >;
    if ($filename eq '')
    {
      $filename = < $config{'clamav_rcd_path'}/rc.local/{K,S}*$freshclambase >;
      $type_other = 1 if ($filename ne '');
    }

    if ($filename ne '')
    {
      $base = basename ($filename);
      $base =~ /^.([0-9]+).*$/;
      $order = $1;

      if ($type_other == 0)
      {
        system (
          "$all_path{'mv'}mv -f /etc/rc$runlevel.d/S$order$freshclambase " .
          "/etc/rc$runlevel.d/K$order$freshclambase");
      }
      else
      {
         system (
          "$all_path{'mv'}mv -f /etc/rc.local/S$order$freshclambase " .
          "/etc/rc.local/K$order$freshclambase");
      }
    }
  }
  
  &clamav_delete_cron ('update');
}

# clamav_bsd_update_state ( $ $ )
# IN: daemon name (freshclam or clamav)
#     state (YES or NO)
# OUT: -
#
# Set the system state for a given daemon on BSD systems
# 
sub clamav_bsd_update_state ( $ $ )
{
  my ($daemon, $state) = @_;
  my $found = 0;

  system ("$all_path{'cp'}cp -f /etc/rc.conf /etc/rc.conf_clamav");

  open (SRC, "</etc/rc.conf_clamav");
  open (DST, ">/etc/rc.conf"); 
  while (my $line = <SRC>)
  {
    if ($line =~ /clamav_${daemon}_enable/)
    {
      $line = "clamav_${daemon}_enable=\"$state\"\n";
      $found = 1;
    }

    print DST $line;
  }

  print DST "clamav_${daemon}_enable=\"$state\"\n"
    if (!$found);

  close (DST);
  close (SRC);
}

# clamav_delete_cron ( $ )
# IN: type of the cron (purge|update)
# OUT: -
#
# Delete the line associated with the given type in
# /etc/cron.d/webmin_clamav or /etc/crontab (bsd)
# 
sub clamav_delete_cron ( $ )
{
  my $type = shift;
  my $path = '';
  my $path_sav = '';

  $path = (-d "/etc/cron.d/") ?
    "/etc/cron.d/webmin_clamav" : "/etc/crontab";
  $path_sav = "$path.sav";

  system ("$all_path{'cp'}cp -f $path $path_sav");

  open (SRC, "<$path_sav");
  open (DST, ">$path"); 
  while (my $line = <SRC>)
  {
    print DST $line
      if ($line !~ /^.*#.*$type/);
  }
  close (DST);
  close (SRC);
}

# clamav_set_freshclam_daemon_settings ( $ $ )
# IN: old frequency, new frequency
# OUT: -
#
# Update the check interval frequency of the refresh for the
# freshclam daemon and restat it
# 
sub clamav_set_freshclam_daemon_settings ( $ $ )
{
  my ($oldfreq, $freq) = @_;
  my $runlevel = '';
  my $freshclam = '';
  my $freshclambase = '';
  my $type_other = 0;
  my $bsd_clam_state = 'NO';

  $runlevel = &clamav_get_runlevel ();
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
  $bsd_clam_state = &clamav_bsd_get_state ('freshclam')
      if ($gconfig{'os_type'} =~ /bsd/);
 
 # debian
 if ($gconfig{'os_type'} =~ /debian/)
 {
   system ("$all_path{'ln'}ln -s $freshclam /etc/network/if-up.d/");
   system ("$all_path{'ln'}ln -s $freshclam /etc/network/if-down.d/");
 }
 # bsd
 elsif ($gconfig{'os_type'} =~ /bsd/)
 {
   &clamav_bsd_update_state ('freshclam', 'YES');
   system ("$config{'clamav_freshclam_init_script'} start");
 }
 # others
 else
 {
   $filename = 
     < $config{'clamav_rcd_path'}/rc$runlevel.d/{K,S}*$freshclambase >;
   if ($filename eq '')
   {
     $filename =
       < $config{'clamav_rcd_path'}/rc.local/{K,S}*$freshclambase >;
     $type_other = 1 if ($filename ne '');
   }
   $base = basename ($filename);
   $base =~ /^.([0-9]+).*$/;
   $order = $1;

   if ($type_other == 0)
   {
     system (
        "$all_path{'mv'}mv -f " .
        "$config{'clamav_rcd_path'}/rc$runlevel.d/K$order$freshclambase " . 
        "$config{'clamav_rcd_path'}/rc$runlevel.d/S$order$freshclambase"
      );
    }
    else
    {
      system (
        "$all_path{'mv'}mv -f " .
        "$config{'clamav_rcd_path'}/rc.local/K$order$freshclambase " . 
        "$config{'clamav_rcd_path'}/rc.local/S$order$freshclambase"
      );
    }
  }
  
  &clamav_load_freshclam_config ();
  $freshclam_config{'Checks'} = $freq;
  &clamav_save_freshclam_config ();

  system ("$freshclam restart");
}

# clamav_save_freshclam_config ()
# IN: -
# OUT: -
#
# Save the freshclam configuration in freshclam config file
# 
sub clamav_save_freshclam_config
{
  &lock_file ($config{'clamav_freshclam_conf'});
  open (H, ">$config{'clamav_freshclam_conf'}");
  foreach my $key (keys %freshclam_config)
  {
    next if $freshclam_config{$key} eq "$text{'UNDEFINED'}";
    next if ($freshclam_config{$key} eq '' and 
      $freshclam_predefined{$key} == 1);
    
    print H "$key $freshclam_config{$key}\n";
  }
  close (H);
  &unlock_file ($config{'clamav_freshclam_conf'});
}

# clamav_save_global_settings ()
# IN: -
# OUT: -
#
# Save ClamAV/Freshclam settings in config files
# 
sub clamav_save_global_settings
{
  my $restart = shift;
  my $cc = '';
  my $fc = '';
  
  $cc = $config{'clamav_clamav_conf'};
  $fc = $config{'clamav_freshclam_conf'};

  if (not $restart)
  {
    $cc = "/tmp/.clamav/clamav.conf";
    $fc = "/tmp/.clamav/freshclam.conf";
  }
  
  &lock_file ($cc); open (CLAMAV, ">$cc");
  &lock_file ($fc); open (FRESHCLAM, ">$fc");
  
  foreach $key (sort keys %in)
  {
    $value = $key;

    next if ($restart and $in{$key} eq "$text{'UNDEFINED'}");
    
    if ($key =~ /^clamav.*$/)
    {
      $value =~ s/^clamav_//;
      next if ($restart and ($in{$key} eq '' and 
        $clamav_predefined{$value} == 1));
      print CLAMAV "$value $in{$key}\n";
    }
    elsif ($key =~ /^freshclam.*$/)
    {
      $value =~ s/^freshclam_//;
      next if ($restart and ($in{$key} eq '' and 
        $freshclam_predefined{$value} == 1));
      print FRESHCLAM "$value $in{$key}\n";
    }
  }
  
  close (CLAMAV); &unlock_file ($cc);
  close (FRESHCLAM); &unlock_file ($fc);

  if ($restart)
  {
    &clamav_activate_clamd ();
    
    system ("$config{'clamav_freshclam_init_script'} restart")
      if ($config{'clamav_refresh_use_cron'} eq '0');

    &clamav_clean_global_settings_tempfiles ();
  }
}

# clamav_global_settings_get_delete_item ()
# IN: type of item to process ('clamav' or 'freshclam')
# OUT: keys of the item to delete (clamav predefine variable name)
#
# Return the name of the predefined variable to delete in clamav or
# freshclam's config file
# 
sub clamav_global_settings_get_delete_item
{
  my $type = shift;
  my $exp = '';
  my $key = '';

  $exp = 'ns' . $type . '_delete_';

  foreach $key (keys %in)
  {
    if ($key =~ /^$exp/)
    {
      $key =~ s/^$exp//g;
      
      return $key;
    }
  }

  return $key;
}

# clamav_clean_global_settings_tempfiles ()
# IN: -
# OUT: -
#
# Remove temporary files for global settings section
# 
sub clamav_clean_global_settings_tempfiles
{
  unlink ("/tmp/.clamav/clamav.conf");
  unlink ("/tmp/.clamav/freshclam.conf");
}

# clamav_display_clamav_settings ()
# IN: -
# OUT: -
#
# Load clamav configuration file and display it
# 
sub clamav_display_clamav_settings
{
  my ($newkey, $deletekey) = @_;
  my $key = '';
  
  &clamav_load_clamav_config ();
  
  if ($deletekey or $newkey)
  {
    # delete the specified key if it has been specified
    delete $in{"clamav_$deletekey"} if ($deletekey);

    # add a new key in the config file
    $in{"clamav_$newkey"} = 
      ($clamav_predefined{$newkey} == 1) ? "$text{'UNDEFINED'}" : '' 
        if ($newkey);
    
    # save and reload config
    &clamav_save_global_settings (0);
    &clamav_load_clamav_config ();
  }
  
  if (&clamav_display_combo_clamav_predefined (1) == 1)
  {
    print qq( 
      <input type="submit" name="nsclamav_add" value="$text{'ADD_KEY'}">
    );
  }
  
  print qq(<table border=1>);
  foreach $key (sort keys %clamav_config)
  {
    next if ($key eq '');
   
    if ($clamav_config{$key} eq '')
    {
      print qq(
        <tr><td $cb>$key</td>
        <input type="hidden" name="clamav_$key">
	<td><font color="silver"><i>$text{'NO_VALUE'}</i></font></td>
      );
    }
    elsif ($clamav_config{$key} eq "$text{'UNDEFINED'}")
    {
      print qq(
        <tr><td bgcolor="gray"><font color="white"><b>$key</b></font></td>
        <td><input type="text" name="clamav_$key" size=40
        value="$clamav_config{$key}"></td>
      );
    }
    else
    {
      print qq(
        <tr><td $cb>$key</td>
        <td><input type="text" name="clamav_$key" size=40
        value="$clamav_config{$key}"></td>
      );
    }
    print qq(
      <td><input title="$text{'DELETE_ITEM'}" type="submit"
      name="nsclamav_delete_$key"
      value="$text{'DELETE'}"></td>
      </tr>
    );
  }
  print qq(</table>);
}

# clamav_display_freshclam_settings ()
# IN: -
# OUT: -
#
# Load freshclam configuration file and display it
# 
sub clamav_display_freshclam_settings
{
  my ($newkey, $deletekey) = @_;
  my $key = '';
  
  &clamav_load_freshclam_config ();
  
  if ($deletekey or $newkey)
  {
    # delete the specified key if it has been specified
    delete $in{"freshclam_$deletekey"} if ($deletekey);

    # add a new key in the config file
    $in{"freshclam_$newkey"} = 
      ($freshclam_predefined{$newkey} == 1) ? "$text{'UNDEFINED'}" : '' 
        if ($newkey);
    
    # save and reload config
    &clamav_save_global_settings (0);
    &clamav_load_freshclam_config ();
  }
  
  if (&clamav_display_combo_freshclam_predefined (1) == 1)
  {
    print qq( 
      <input type="submit" name="nsfreshclam_add" value="$text{'ADD_KEY'}">
    );
  }
  
  print qq(<table border=1>);
  foreach $key (sort keys %freshclam_config)
  {
    next if ($key eq '');
   
    if ($freshclam_config{$key} eq '')
    {
      print qq(
        <tr><td $cb>$key</td>
        <input type="hidden" name="freshclam_$key">
	<td><font color="silver"><i>$text{'NO_VALUE'}</i></font></td>
      );
    }
    elsif ($freshclam_config{$key} eq "$text{'UNDEFINED'}")
    {
      print qq(
        <tr><td bgcolor="gray"><font color="white"><b>$key</b></font></td>
        <td><input type="text" name="freshclam_$key" size=40
        value="$freshclam_config{$key}"></td>
      );
    }
    else
    {
      print qq(
        <tr><td $cb>$key</td>
        <td><input type="text" name="freshclam_$key" size=40
        value="$freshclam_config{$key}"></td>
      );
    }
    print qq(
      <td><input title="$text{'DELETE_ITEM'}" type="submit"
      name="nsfreshclam_delete_$key"
      value="$text{'DELETE'}"></td>
      </tr>
    );
  }
  print qq(</table>);
}

# clamav_load_clamav_config ()
# IN: -
# OUT: -
#
# Load the clamav config file in memory
# 
sub clamav_load_clamav_config
{
  my $cc = '';
  
  %clamav_config = undef;
  
  $cc = $config{'clamav_clamav_conf'};
  $cc = "/tmp/.clamav/clamav.conf" if (-f "/tmp/.clamav/clamav.conf");
  
  open (H, "<$cc");
  while (<H>)
  {
    next if /#/;

    (my $key, my $value) = split (' ');
    $clamav_config{$key} = $value;
  }
  close (H);
}

# clamav_load_freshclam_config ()
# IN: -
# OUT: -
#
# Load the freshclam config file in memory
# 
sub clamav_load_freshclam_config
{
  my $fc = '';
  
  %freshclam_config = undef;
  
  $fc = $config{'clamav_freshclam_conf'};
  $fc = "/tmp/.clamav/freshclam.conf" if (-f "/tmp/.clamav/freshclam.conf");
  
  open (H, "<$fc");
  while (<H>)
  {
    next if /#/;

    (my $key, my $value) = split (' ');
    $freshclam_config{$key} = $value;
  }
  close (H);
}

# clamav_set_cron_update ( $ $ )
# IN: cron hour, cron day
# OUT: -
#
# Set the cron line for the db refresh in /etc/cron.d/webmin_clamav
# 
sub clamav_set_cron_update ( $ $ )
{
  my ($hour, $day) = @_;
  my $new_line = '';
  my $datadir = '';
  my $path = '';
  my $path_sav = '';

  $path = (-d "/etc/cron.d/") ?
    "/etc/cron.d/webmin_clamav" : "/etc/crontab";
  $path_sav = "$path.sav";

  system ("$all_path{'cp'}cp -f $path $path_sav");

  $datadir = dirname ($config{'clamav_db1'});

  $day = '*' if ($day == 7);
  &clamav_delete_cron ('update');
  $new_line = 
    "$hour\t*\t*\t*\t$day\troot\t\$($all_path{'freshclam'}freshclam " .
    "--quiet " .
    "--config-file $config{'clamav_freshclam_conf'} " .
    "--log $config{'clamav_freshclam_log'} " .
    "--datadir $datadir) " .
    "#update";

  open (H, ">>$path");
  print H "$new_line\n";
  close (H);
}

# clamav_get_logfiles ()
# IN: -
# OUT: A array with logfiles
#
# Identify ClamAV log files and fill a array with them
#
sub clamav_get_logfiles
{
  @ret = ();

  foreach $key (%config)
  {
    push @ret, $key if ($key =~ /^.*\.log$/);
  }

  return @ret;
}

# clamav_set_cron_purge ( $ $ )
# IN: cron hour, cron day
# OUT: -
#
# Set cron line for the quarantine repository purge
# 
sub clamav_set_cron_purge ( $ $ )
{
  my ($hour, $day) = @_;
  my $new_line = '';
  my $path = '';
  my $path_sav = '';

  $path = (-d "/etc/cron.d/") ?
    "/etc/cron.d/webmin_clamav" : "/etc/crontab";
  $path_sav = "$path.sav";

  system ("$all_path{'cp'}cp -f $path $path_sav");

  $day = '*' if ($day == 7);
  &clamav_delete_cron ('purge');
  $new_line = 
    "$hour * * * $day root \$($all_path{'rm'}rm -f " .
    "$config{'clamav_quarantine'}/*.{msg,log}) #purge";

  open (H, ">>$path");
  print H "$new_line\n";
  close (H);
}

# clamav_update_db ()
# IN: -
# OUT: -
#
# Update virus signatures databases displaying output on the current web
# page
# 
sub clamav_update_db
{
  print qq(<div><pre style="background: #cccccc">);;
  open (H, "$all_path{'freshclam'}freshclam 2>&1 |");
  while (<H>)
  {
    s/^*\n$/<br>/g;
    if (/load/)
    {
      print '.';
    }
    else
    {
      print;
    };
  }
  close (H);
  print qq(</pre>);
}

# clamav_get_cron_settings ( $ )
# IN: type of the cron (purge|update)
# OUT: a array representing a cron line
#
# Return a array of a cron line corresponding to the given type. if line
# is not founded, return undef
# 
sub clamav_get_cron_settings ( $ )
{
  my $type = shift;
  my @lines = ();
  my @cron_line = ();
  my $path = '';
  my $path_sav = '';

  $path = (-d "/etc/cron.d/") ?
    "/etc/cron.d/webmin_clamav" : "/etc/crontab";
  $path_sav = "$path.sav";

  system ("$all_path{'cp'}cp -f $path $path_sav");

  return undef 
    if (! -e "$path");

  open (H, "<$path");
  @lines = <H>;
  close (H);
                                                                                
  foreach (@lines)
  {
    @cron_line = 
      (/^(\w+|\*)\s+(\w+|\*)\s+(\w+|\*)\s+(\w+|\*)\s+(\w+|\*)\s+(\w+)\s+(.*)\s+(#\w+)$/);
                                                                                
    return @cron_line
      if ($8 =~ /$type/);
  }
                                                                                
  return undef;
}

# clamav_freshclam_daemon_settings_table ( $ )
# IN: frequency
# OUT: a buffer to display
#
# Build a HTML table with all possible frequencies for freshclam
# daemon
# 
sub clamav_freshclam_daemon_settings_table ( $ )
{
  my $freq = shift;

  $buffer .= qq(
    <table border="1">
    <tr $tb><td><b>$text{'FREQUENCY'}</b></td></tr>
    <tr $cb>
    <td><select name="freq">
  );

  foreach (1..50)
  {
    $default = ($_ == $freq) ? ' SELECTED' : '';
    $buffer .= qq(<option value="$_"$default>$_</option>\n);
  }

  $buffer .= qq(</select></td></tr></table>);

  return $buffer;
}

# clamav_cron_settings_table ( $ $ )
# IN: default cron hour, default cron day
# OUT: a buffer to display
#
# Build a HTML table for choose hour a day for execution of refresh
# db cron
#
sub clamav_cron_settings_table
{
  my ($hour, $day) = @_;
  my $buffer = '';

  $buffer .= qq(
    <table border="1">
    <tr $tb><td><b>$text{'HOUR'}</b></td><td><b>$text{'DAY'}</b></td></tr>
    <tr $cb>
    <td><select name="hour">
  );

  foreach (0..23)
  {
    $default = ($_ == $hour) ? ' SELECTED' : '';
    $buffer .= qq(<option value="$_"$default>$_</option>\n);
  }

  $buffer .= qq(</select></td><td><select name="day">);

  $day = 7 if ($day eq '*');
  $default = ($day == 7) ? ' SELECTED' : '';
  $buffer .= qq(<option value="7"$default>$text{'EVERYDAY'}</option>\n);
  for (my $i = 0; $i < 7; $i++)
  {
    my $value = $days[$i];
    my $default = ($i == $day) ? ' SELECTED' : '';
                                                                                
    $buffer .= qq(<option value="$i"$default>$value</option>\n);
  }

  $buffer .= qq(</select></td></tr></table>);

  return $buffer;
}

# clamav_is_quarantine_repository_empty ()
# IN: -
# OUT: true if empty
#
# Check wether the quarantine repository is empty or not.
# 
sub clamav_is_quarantine_repository_empty
{
  my @files = ();

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir(DIR);
  closedir (DIR);

  return (($#files - 1) <= 0);
}

# clamav_cut_string ( $ $ )
# IN: string, max length
# OUT: a cutted string
#
# Cut a string to fit in the given length, adding "[..]" at
# the end
# 
sub clamav_cut_string ( $ $ )
{   
  my ($string, $max_len) = @_;

  $string = (substr ($string, 0, $max_len - 5)) . ' [..]'
    if (($max_len > 5) and (length ($string) >= $max_len));
                                                                                
  return $string;
}

# clamav_is_clamd_installed ()
# IN: -
# OUT: -
#
# Check if specified clamd path is ok on the system
# 
sub clamav_is_clamd_installed
{
  return (-e $config{'clamav_init_script'});
}

# clamav_activate_clamd ()
# IN: -
# OUT: -
#
# Restart the clamav daemon, waiting on second for process
# to be up
# 
sub clamav_activate_clamd
{
  system ("$config{'clamav_init_script'} restart");
  sleep (1);
}

# clamav_deactivate_clamd ()
# IN: -
# OUT: -
#
# Stop the clamav daemon, waiting one second for process
# to be down
# 
sub clamav_deactivate_clamd
{
  system ("$config{'clamav_init_script'} stop");
  sleep (1);
}

# clamav_is_clamd_alive ()
# IN: -
# OUT: -
#
# Check wether or not clamd daemon is alive
# 
sub clamav_is_clamd_alive
{
  return &find_byname ($config{'clamav_daemon_name'});
}

# clamav_get_email_header_values ( $ $ )
# IN: email file name on the disk, field to retreive value for
# OUT: a array with requested values
#
# Search and return values in a array for the requested fields,
# in the inverse order than requested
# 
sub clamav_get_email_header_values ( $ $ )
{
  my ($file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;
 
  if (open (H, $file))
  {
    LOOP: while (defined ($item = <H>) and ($i <= $#names))
    {
      $item =~ s/^[ ,\t]*//g;

      foreach my $name (@names)
      {
        if ($item =~ /^$name: .*\n$/)
        {
          $i++;
          $item =~ s/^$name: //;
          $header{$name} = $item;
	  next LOOP;
        }
      }
    }
    close (H);
  }
                                                                                
  return %header;
}

# clamav_get_log_values ( $ $ )
# IN: log file name on the disk, field to retreive value for
# OUT: a array with requested values
#
# Search and return values in a array for the requested fields,
# in the inverse order than requested
#
sub clamav_get_log_values ( $ $ )
{
  my ($file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;
                                                                                
  if (open (H, $file))
  {
    while (defined ($item = <H>) and ($i <= $#names))
    {
      foreach my $name (@names)
      {
        if ($item =~ /: $name: .*\n$/)
        {
          $i++;
          $item =~ s/^.*: $name:.*: //;
          $header{$name} = $item;
        }
      }
    }
    close (H);
  }
                                                                                
  return %header;
}

# clamav_print_email ( $ )
# IN: email file on the disk
# OUT: -
#
# Display the content of the given mail file
# 
sub clamav_print_email ( $ )
{
  my $file = shift;
  my @content = ();
                                                                                
  exit (1) if ($file =~ /\// or $file =~ /\.\./ or $file =~ /;/);
    
  # is amavisd-new installed or amavis-ng
  $file = ($file =~ /^virus/) ? $file : "$file.msg";
    
  open (H, "$config{'clamav_quarantine'}/$file");
  @content = <H>;
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_print_email_infos ( $ )
# IN: email file on the disk
# OUT: -
#
# Display some email information in a HTML table
# 
sub clamav_print_email_infos ( $ )
{
  my $base = shift;
  my %header = ();
  my $subject = '';
  my $from = '';
  my $to = '';

  # if amavisd-new is installed
  if ($base =~ /^virus/)
  {
    %header = &clamav_get_email_header_values (
      "$config{'clamav_quarantine'}/$base",
      qw(Subject From To)
    );
    $subject = $header{'Subject'};
    $from = $header{'From'};
    $to = $header{'To'};
  }
  # if amavis-ng is installed
  else
  {
    %header = &clamav_get_email_header_values (
      "$config{'clamav_quarantine'}/$base.msg",
      qw(Subject X-Quarantined-From X-Quarantined-To)
    );
    $subject = $header{'Subject'};
    $from = $header{'X-Quarantined-From'};
    $to = $header{'X-Quarantined-To'};
  }

  print qq(
    <p><table border=0>
    <tr><td $cb><b>$text{'SUBJECT'}: </b></td><td>$subject</td><tr>
    <tr><td $cb><b>$text{'FROM'}: </b></td><td>$from</td><tr>
    <tr><td $cb><b>$text{'TO'}: </b></td><td>$to</td><tr>
    </table></p>
  );
}

# clamav_resend_email ( $ )
# IN: email file on the disk
# OUT: -
#
# Resend a false positive email. it first test if there is an
# existing amavis-inject script on the server. if not, it use
# it is proper copy of this perl script (./bin/amavis-inject).
#
# supported MTA are:
#
#   - rsmtp
#   - sendmail
# 
sub clamav_resend_email ( $ )
{
  my $base = shift;
  my $amavis = "$all_path{'amavis-inject'}amavis-inject ";
  my $mta = " $all_path{'sendmail'}sendmail -bS "; # use sendmail by default

  return if ($base =~ /\// or $base =~ /\.\./ or $base =~ /;/);
  
  # if rsmtp is found, use it instead of sendmail
  $mta = " $all_path{'rsmtp'}rsmtp " if ($all_path{'rsmtp'} ne '');
  # if amavis-inject not found, use our own copy  
  $amavis = "./bin/amavis-inject " if ($all_path{'amavis-inject'} eq '');

  # if amavisd-new is installed
  if ($base =~ /^virus/)
  {
    system ("$amavis $config{'clamav_quarantine'}/$base | $mta");
  }
  # if amavis-ng is installed
  else
  {
    system ("$amavis $config{'clamav_quarantine'}/$base.msg | $mta");
  }
}

# clamav_remove_email ( $ )
# IN: base name of the email file on the disk
# OUT: -
#
# Delete a email and its associated log file on the quarantine
# repository
# 
sub clamav_remove_email
{
  my $base = shift;

  exit (1) if ($base =~ /\// or $base =~ /\.\./ or $base =~ /;/);
  
  # if amavisd-new is installed
  if ($base =~ /^virus/)
  {
    system("$all_path{'rm'}rm -f $config{'clamav_quarantine'}/$base");
  }
  # if amavis-ng is installed
  else
  {
    system("$all_path{'rm'}rm -f $config{'clamav_quarantine'}/$base.*");
  }
}

# clamav_print_log ( $ $ )
# IN: file to display, number of lines to display
# OUT: -
#
# Display the content of the given log file
#
sub clamav_print_log
{
  my $file = shift;
  my $lines = shift;
  my @content = ();

  exit (1) if ($file =~ /;/ or $lines =~ /;/);
  
  open (H, "$file");
  if ($lines eq $text{'ALL'})
  {
    @content = <H>;
  }
  else
  {
    @content = `/usr/bin/tail -n $lines $file`;
  }
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_quarantine_print_log ( $ )
# IN: email log file on the disk
# OUT: -
#
# Display the content of the given mail log file
#
sub clamav_quarantine_print_log ( $ )
{
  my $file = shift;
  my @content = ();

  exit (1) if ($file =~ /\// or $file =~ /\.\./ or $file =~ /;/);
    
  open (H, "$config{'clamav_quarantine'}/$file.log");
  @content = <H>;
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_print_quarantine_table_amavis_ng ( $ $ $ $ $ $ $ $ $ )
# IN: -
# OUT: -
#
# -- used if amavis-ng has been found --
# 
# Display a HTML table containing on line per email found in the quarantine
# repository
# 
sub clamav_print_quarantine_table_amavis_ng ( $ $ $ $ $ $ $ $ $ $ )
{
  my ($total, $current, $virus_name, $mail_from, $mail_to,
      $day1, $month1, $year1,
      $day2, $month2, $year2) = @_;
  my @files = ();
  my $i = 0;
  my $STRING_MAX_LEN = 30;
  my $date1 = '';
  my $date2 = '';
  my $page_count = 0;
  my $count = 0;
  my $total = 0;
  my $arows = ();

  $current = int ($current);
  $total = int ($total);

  $date1 = &clamav_format_date ($day1, $month1, $year1);
  $date2 = &clamav_format_date ($day2, $month2, $year2);
  if ($date1 ne '' and $date2 eq '')
    {$date2 = '2050-12-30';}
  elsif ($date2 ne '' and $date1 eq '')
    {$date1 = '2050-12-30';}

  $virus_name =~ s/ //g;
  $mail_from =~ s/ //g;
  $mail_to =~ s/ //g;

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir (DIR);
  closedir (DIR);
  @files = sort @files;

  # browse result and push it in a array to manage
  # pagination
  $count = 0;
  for ($i = 0; $i < $#files + 1; $i += 2)
  {
    next if ($files[$i] eq '.');
      
    my $log = $files[$i];
    my $msg = $files[$i + 1];
    my $quarantine_path = $config{'clamav_quarantine'};
    my $base = $log; $base =~ s/\..*$//g;
    my %header = &clamav_get_email_header_values (
        "$quarantine_path/$base.msg",
        qw(Subject X-Quarantined-From X-Quarantined-To)
      );
    my $subject = $header{'Subject'};
    my $from = $header{'X-Quarantined-From'};
    my $to = $header{'X-Quarantined-To'};
    my %header = &clamav_get_log_values (
        "$quarantine_path/$base.log",
        qw(AMAVIS::AV::CLAMD)
      );
    my $virus = $header{'AMAVIS::AV::CLAMD'};
    my $url_virus = &urlize ($virus);
    my $fdate = '';

    # applying filters
    next if ($virus_name ne '') and $virus !~ /$virus_name/i;
    next if ($mail_from ne '') and $from !~ /$mail_from/i;
    next if ($mail_to ne '') and $to !~ /$mail_to/i;

    open (H, "$quarantine_path/$base.msg");
    $fdate = strftime ("%Y-%m-%d", localtime ((stat (H))[9]));
    close (H);
    
    next if (
      $date1 ne '' and
      (&Date_Cmp ($fdate, $date1) < 0 or
      &Date_Cmp ($fdate, $date2) > 0)
    );

      $count++;

      my %hrow = ();
      $hrow{'date'} = $fdate;
      $hrow{'subject'} = $subject;
      $hrow{'from'} = $from;
      $hrow{'to'} = $to;
      $hrow{'virus'} = $virus;
      $hrow{'url_virus'} = $url_virus;
      $hrow{'base'} = $base;

      push (@arows, \%hrow);
  }
  
  # return if no result
  if ($count <= 0)
  { 
    print qq(<p><b>$text{'NO_RESULT_QUARANTINE'}</b></p>);
    return;
  }

  print qq(
    <table border>
    <tr $tb><td><b>$text{'SUBJECT'}</b></td>
    <td><b>$text{'VIRUS'}</b></td>
    <td><b>$text{'FROM'}</b></td><td><b>$text{'TO'}</b></td>
    <td colspan=3><b>$text{'ACTION'}</b></td></tr>
  );

  # display search result
  for ($i = ceil ($current * $PAGE_ITEMS); 
    $i < $#arows + 1 and $page_count++ < $PAGE_ITEMS; $i++)
  {
    my %hrow = %{$arows[$i]};

    my $subjectc = &clamav_cut_string ($hrow{'subject'}, $STRING_MAX_LEN);
    my $fromc = &clamav_cut_string ($hrow{'from'}, $STRING_MAX_LEN);
    my $toc = &clamav_cut_string ($hrow{'to'}, $STRING_MAX_LEN);
    my $virusc = &clamav_cut_string ($hrow{'virus'}, $STRING_MAX_LEN);
    
      print qq(
        <tr>
        <td $cb><a href="quarantine_viewmail.cgi?base=$hrow{'base'}" title="$text{'DATE'}: $hrow{'date'}\n$text{'FROM'}: $hrow{'from'}$text{'TO'}: $hrow{'to'}$text{'VIRUS'}: $hrow{'virus'}\n$text{'SUBJECT'}: $hrow{'subject'}">
        $subjectc</a></td>
        <td $cb><b><a href="/clamav/vdb_search_main.cgi?search=on&virus=$hrow{'url_virus'}" title="$text{'VIRUS'}: $hrow{'virus'}">$virusc</a></b></td>
        <td $cb title="$text{'FROM'}: $hrow{'from'}">$fromc</td>
        <td $cb title="$text{'TO'}: $hrow{'to'}">$toc</td>
        <td $cb><a href="quarantine_viewlog.cgi?base=$hrow{'base'}">
        $text{'VIEWLOG'}</a></td>
        <td $cb><a href="quarantine_resend.cgi?base=$hrow{'base'}">
        $text{'RESEND'}</a></td>
        <td $cb><a href="quarantine_remove.cgi?base=$hrow{'base'}">
        $text{'DELETE'}</a></td>
        </tr>
      );
  }

  print qq(</table>);

  return $count;
}

# clamav_print_quarantine_table_amavisd_new ( $ $ $ $ $ $ $ $ $ )
# IN: -
# OUT: -
# 
# -- used if amavisd-new has been found --
#
# Display a HTML table containing on line per email found in the quarantine
# repository
# 
sub clamav_print_quarantine_table_amavisd_new ( $ $ $ $ $ $ $ $ $ $ $ )
{
  my ($total, $current, $virus_name, $mail_from, $mail_to,
      $day1, $month1, $year1,
      $day2, $month2, $year2) = @_;
  my @files = ();
  my $i = 0;
  my $STRING_MAX_LEN = 30;
  my $date1 = '';
  my $date2 = '';
  my $page_count = 0;
  my $count = 0;
  my @arows = ();

  $current = int ($current);
  $total = int ($total);

  $date1 = &clamav_format_date ($day1, $month1, $year1);
  $date2 = &clamav_format_date ($day2, $month2, $year2);
  if ($date1 ne '' and $date2 eq '')
    {$date2 = '2050-12-30';}
  elsif ($date2 ne '' and $date1 eq '')
    {$date1 = '2050-12-30';}

  $virus_name =~ s/ //g;
  $mail_from =~ s/ //g;
  $mail_to =~ s/ //g;

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir (DIR);
  closedir (DIR);
  @files = sort @files;

  # browse result and push it in a array to manage
  # pagination
  $count = 0;
  for ($i = 0; $i < $#files + 1; $i++)
  {
      next if ($files[$i] !~ /^virus/);
      
      my $msg = $files[$i];
      my $base = $msg;
      my $quarantine_path = $config{'clamav_quarantine'};
      my %header = &clamav_get_email_header_values (
          "$quarantine_path/$msg",
          qw(Subject From To X-AMaViS-Alert)
        );
      my $subject = $header{'Subject'};
      my $from = $header{'From'};
      my $to = $header{'To'};
      my $virus = $header{'X-AMaViS-Alert'};
      ($virus) = (split (':', $virus))[1];
      $virus =~ s/[ ,\n,\r]//g;
      my $url_virus = &urlize ($virus);
      my $fdate = '';

      # applying filters
      next if ($virus_name ne '') and $virus !~ /$virus_name/i;
      next if ($mail_from ne '') and $from !~ /$mail_from/i;
      next if ($mail_to ne '') and $to !~ /$mail_to/i;
    
      open (H, "$quarantine_path/$msg");
      $fdate = strftime ("%Y-%m-%d", localtime ((stat (H))[9]));
      close (H);
    
      next if (
        $date1 ne '' and
        (&Date_Cmp ($fdate, $date1) < 0 or
        &Date_Cmp ($fdate, $date2) > 0)
      );

      $count++;

      my %hrow = ();
      $hrow{'date'} = $fdate;
      $hrow{'subject'} = $subject;
      $hrow{'from'} = $from;
      $hrow{'to'} = $to;
      $hrow{'virus'} = $virus;
      $hrow{'url_virus'} = $url_virus;
      $hrow{'base'} = $base;

      push (@arows, \%hrow);
  }

  # return if no result
  if ($count <= 0)
  { 
    print qq(<p><b>$text{'NO_RESULT_QUARANTINE'}</b></p>);
    return;
  }

  print qq(
    <table border>
    <tr $tb><td><b>$text{'SUBJECT'}</b></td>
    <td><b>$text{'VIRUS'}</b></td>
    <td><b>$text{'FROM'}</b></td><td><b>$text{'TO'}</b></td>
    <td colspan=2><b>$text{'ACTION'}</b></td></tr>
  );

  # display search result
  for ($i = ceil ($current * $PAGE_ITEMS); 
    $i < $#arows + 1 and $page_count++ < $PAGE_ITEMS; $i++)
  {
    my %hrow = %{$arows[$i]};

    my $subjectc = &clamav_cut_string ($hrow{'subject'}, $STRING_MAX_LEN);
    my $fromc = &clamav_cut_string ($hrow{'from'}, $STRING_MAX_LEN);
    my $toc = &clamav_cut_string ($hrow{'to'}, $STRING_MAX_LEN);
    my $virusc = &clamav_cut_string ($hrow{'virus'}, $STRING_MAX_LEN);
    
      print qq(
        <tr>
        <td $cb><a href="quarantine_viewmail.cgi?base=$hrow{'base'}" title="$text{'DATE'}: $hrow{'date'}\n$text{'FROM'}: $hrow{'from'}$text{'TO'}: $hrow{'to'}$text{'VIRUS'}: $hrow{'virus'}\n$text{'SUBJECT'}: $hrow{'subject'}">
        $subjectc</a></td>
        <td $cb><b><a href="/clamav/vdb_search_main.cgi?search=on&virus=$hrow{'url_virus'}" title="$text{'VIRUS'}: $hrow{'virus'}">$virusc</a></b></td>
        <td $cb title="$text{'FROM'}: $hrow{'from'}">$fromc</td>
        <td $cb title="$text{'TO'}: $hrow{'to'}">$toc</td>
        <td $cb><a href="quarantine_resend.cgi?base=$hrow{'base'}">
        $text{'RESEND'}</a></td>
        <td $cb><a href="quarantine_remove.cgi?base=$hrow{'base'}">
        $text{'DELETE'}</a></td>
        </tr>
      );
  }

  print qq(</table>);

  return $count;
}

# clamav_format_date ( $ $ $ )
# IN: The days, month and year
# OUT: The formated date (YYYY-MM-DD)
#
# Format a date
# 
sub clamav_format_date ( $ $ $ )
{
  my ($day, $month, $year) = @_;
  
  $day = int ($day);
  $month = int ($month);
  $year = int ($year);

  return '' if ($day == 0 || $month == 0 || $year == 0);

  $month++;
  $year += 2000 if ($year < 1000);

  $date = 
    sprintf ("%u-%02u-%02u", $year, $month, $day);

  return $date;
}

# clamav_get_months_combo_options ( $ )
# IN: Default choice
# OUT: A buffer containing the SELECT options
#
# Build options for the SELECT tag with months
# 
sub clamav_get_months_combo_options ( $ )
{
  my $default = shift;
  my $i = 0;
  my $buf = '';
  
  foreach (split (' ', $text{'MONTHS_LIST'}))
  {
    $buf .= "<option value=\"$i\"" . (($i == $default) ? ' SELECTED' : '') . 
      ">$_</option>\n";
    $i++;
  }

  return $buf;
}

# clamav_get_period_chooser ( $ $ $ $ $ $ )
# IN: Defaults day1, month1, year1, day2, month2 and year2
# OUT: -
# 
# Display fields to manage preriod
# 
sub clamav_get_period_chooser ( $ $ $ $ $ $ )
{
  my ($day1, $month1, $year1, $day2, $month2, $year2) = @_;
  
  # first period
  print "<table border=0><tr>";
  print qq(<td>$text{'FROM_PERIOD'}</td><td><input type="text" name="day1" size="2" maxlength="2" value="$day1"></td><td><select name="month1">);
  print &clamav_get_months_combo_options ($month1);
  print qq(</select></td><td><input type="text" name="year1" size="4" maxlength="4" value="$year1"></td><td>);
  print &date_chooser_button ('day1', 'month1', 'year1', 1);
  print "</td></tr><tr>";
  # second period
  print qq(<td>$text{'TO_PERIOD'}</td><td><input type="text" name="day2" size="2" maxlength="2" value="$day2"></td><td><select name="month2">);
  print &clamav_get_months_combo_options ($month2);
  print qq(</select></td><td><input type="text" name="year2" size="4" maxlength="4" value="$year2"></td><td>);
  print &date_chooser_button ('day2', 'month2', 'year2', 1);
  print "</td></tr></table>";
}

sub clamav_display_page_panel ( $ $ )
{
  my ($current, $max) = @_;
  my $previous = 0;
  my $next = 0;
  my $i = 0;

  # do we need to paginate?
  return if ($max <= $PAGE_ITEMS);

  $current = int ($current);
  $limit = $max / $PAGE_ITEMS;
  #$limit-- if ($limit % $PAGE_ITEMS);
  $limit = ceil ($limit);
  $previous = $current - 1 if ($current > 0);
  $next = $current + 1;

  $url = 
    "search=1&" .
    "virus_name=$in{'virus_name'}&" .
    "mail_from=$in{'mail_from'}&" .
    "mail_to=$in{'mail_to'}&" .
    "day1=$in{'day1'}&" .
    "day2=$in{'day2'}&" .
    "month1=$in{'month1'}&" .
    "month2=$in{'month2'}&" .
    "year1=$in{'year1'}&" .
    "year2=$in{'year2'}";

  print qq(<p><table border="0" cellspacing="2" cellpadding="2" width="40%">);
  if ($current > 0)
  {
    print qq(<tr><td valign="top" align="center" $cb width="10%">);
    print qq(<a href="quarantine_main.cgi?total=$max&cp=$previous&$url">);
    print qq($text{'PREVIOUS'}</a>);
  }
  else
  {
    print qq(<tr><td valign="top" align="center" width="10%">&nbsp;);
  };
  print qq(</td><td valign="top" align="center" width="80%">);

  for ($i = 0; $i < $limit; $i++)
  {
    if ($i == $current)
    {
      print '<b>' . ($i + 1) . '</b> ';
    }
    else
    {
      print qq(<a href="quarantine_main.cgi?total=$max&cp=$i&$url">);
      print $i + 1;
    }
    print qq(</a> );
  }

  if ($current < $limit - 1)
  {
    print qq(</td><td valign="top" align="center" $cb width="10%">);
    print qq(<a href="quarantine_main.cgi?total=$max&cp=$next&$url">$text{'NEXT'}</a>);
  }
  else
  {
    print qq(</td><td valign="top" align="center" width="10%">&nbsp;);
  };
  print qq(
    </td></tr>
    </table></p>
  );
}
1;
